<?php

/*
 * Copyright (C) 2011 Alejandro Albarca Martínez <albarcam [arroba] gmail.com> and NETFLIE. (http://www.netflie.es)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program, in the file called "COPYING".  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The class is responsible of getting, setting and filtering the possible station sensor values.
 * @name       Sensor
 * @package    PHPWeatherStation
 * @copyright  Copyright (c) 2011 NETFLIE. (http://meteolive.netflie.es)
 * @license    http://www.gnu.org/licenses/     GNU GPLv3
 * @version    $Id: Units.php 2011-10-17 17:00
 * @author     Alejandro Albarca Martinez
 */
class Sensor {

  /**
   * Inside sensor
   * Triple sensors that report temperature, humidity and pressure can have IDs "thb0" to
   * "thb9". The primary indoor sensor should always be "thb0".
   * @var array
   */
  private $_inside;

  /**
   * Outside sensor
   * Sensors that feature temperature and humidity can have IDs from "th0" to "th39".
   * The primary outdoor sensor should always be named "th0" to keep it compatible to
   * the WMR-928 naming.
   * @var array
   */
  private $_outside;

  /**
   * Rain sensor
   * Allowed values: "rain0" to "rain9"
   * @var array
   */
  private $_rain;

  /**
   * Solar radiation sensor
   * Allowed values: "sol0" to "sol9"
   * @var array
   */
  private $_solar;

  /**
   * UV sensor
   * Allowed values: "uv0" to "uv9"
   * @var array
   */
  private $_uv;

  /**
   * Date unit
   * Allowed values: 'local or 'utc'
   * @var array
   */
  private $_wind;

  /**
   * Create a new Sensor object. 
   * @example examples/newSensorObject.example Creating a new Sensor object
   * 
   * @param array $sensors Array of sensors.
   */
  public function __construct($sensors) {
    try {
      if ($sensors) {
        if (isset($sensors[0]) && isset($sensors[1]) && isset($sensors[2]) && isset($sensors[3])
                || isset($sensors['S_INSIDE']) && isset($sensors['S_OUTSIDE']) && isset($sensors['S_RAIN']) && isset($sensors['S_WIND'])) {

          $this->__set('inside', isset($sensors[0]) ? $sensors[0] : $sensors['S_INSIDE']);
          $this->__set('outside', isset($sensors[1]) ? $sensors[1] : $sensors['S_OUTSIDE']);
          $this->__set('rain', isset($sensors[2]) ? $sensors[2] : $sensors['S_RAIN']);
          $this->__set('wind', isset($sensors[3]) ? $sensors[3] : $sensors['S_WIND']);
        } else {
          throw new Exception('Sensor constructor is not valid. Review the documentation.');
        }
      }
    } catch (Exception $e) {
      echo $e;
    }
  }

  /**
   * Get the value of a sensor.
   * 
   * @param array $name Sensor name
   * @return array Sensor value
   */
  public function __get($name) {
    $pattern = '/(date|inside|outside|solar|rain|uv|wind)/';
    preg_match($pattern, $name, $matches);
    
    try {
      switch ($matches[1]) {
        case 'date' :
          return 'date0';
        case 'inside':
          return $this->_inside;
        case 'outside':
          return $this->_outside;
        case 'solar' :
          return $this->_solar;
        case 'rain':
          return $this->_rain;
        case 'uv' :
          return $this->_uv;
        case 'wind':
          return $this->_wind;
        default :
          throw new Exception('The sensor "' . $name . '" does not exist.');
      }
    } catch (Exception $e) {
      echo $e;
    }
  }

  /**
   * Set a value for a sensor.
   * 
   * @param array $name Sensor name
   * @param arry $value Sensor value
   */
  public function __set($name, $value) {
    $name = strtolower($name);
    $value = strtolower($value);

    $nameSensor = '_' . $name; // Private name var

    // If the sensor exists
    try {
      if (property_exists($this, $nameSensor)) {
        // If the sensor value is valid
        if ($this->_isValid($name, $value)) {
          $this->$nameSensor = $value;
        } else {
          throw new Exception('The value: "' . $value . '" for "' . $name . '" sensor, is not valid.');
        }
      } else {
        throw new Exception('The sensor "' . $name . '" does not exist.');
      }
    } catch (Exception $e) {
      echo $e;
    }
  }

  /**
   * Determine if the value for a sensor is valid or not.
   * @param array $name Sensor name
   * @param array $value Sensor value
   * @return bool Return true if the value for the sensor is valid.
   */
  private function _isValid($name, $value) {
    switch ($name) {
      case 'inside':
        $pattern = '/^(thb)[0-9]$/';
        break;
      case 'outside':
        $pattern = '/^(th)[0-39]$/';
        break;
      case 'rain':
        $pattern = '/^(rain)[0-9]$/';
        break;
      case 'solar':
        $pattern = '/^(sol)[0-9]$/';
        break;
      case 'uv':
        $pattern = '/^(uv)[0-9]$/';
        break;
      case 'wind':
        $pattern = '/^(wind)[0-9]$/';
    }

    if (preg_match($pattern, $value))
      return true;

    return false;
  }

}