<?php

/*
 * Station
 * 
 * Copyright (C) 2011-2012 Alejandro Albarca Martínez <albarcam [arroba] gmail.com> and NETFLIE. (http://www.netflie.es)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program, in the file called "COPYING".  If not, see <http://www.gnu.org/licenses/>.
 */

/**
 * The Station class to easily read data from a station connected to Meteohub using an XML file.
 * @name       Station
 * @package    PHPWeatherStation
 * @copyright  Copyright (c) 2011 NETFLIE. <http://meteolive.netflie.es>
 * @license    http://www.gnu.org/licenses/     GNU GPLv3
 * @version    $Id: Station.php 2012-02-04 17:00
 * @author     Alejandro Albarca Martinez
 */
class Station {

  /**
   * DOMXPath object
   *
   * @var DOMXPath
   */
  private $_xpath;

  /**
   * Sensors object
   *
   * @var PHPWeatherStation_Sensors
   */
  private $_sensors;

  /**
   * Units object
   *
   * @var PHPWeatherStation_Units
   */
  private $_units;

  /**
   * Creates a new Station object
   *
   * @param string $xmlDoc The XML document URI
   * @param array $sensors Array of sensor values
   * @see Sensor::__construct()
   * @param array $units  Array of unit values
   * @see Unit::__construct()
   * @example examples/newStationObject.example Creating a new Station object
   */
  public function __construct($xmlDoc, $sensors = null, $units = null) {

    $this->setXPath($xmlDoc);

    require_once 'Sensor.php';
    $this->_sensors = new Sensor($sensors);
    require_once 'Unit.php';
    $this->_units = new Unit($units);
  }

  /**
   * Get Meteohub XML-Data items or Station attributes
   * 
   * @param string $name Tag-name or station attribute
   * @return array Tag-name value or station attribute value
   * @example examples/getStation.example
   */
  public function __get($name) {
    // Make $name lowercase
    $name = strtolower($name);

    $attr = explode('_', $name);
    try {
      switch (sizeof($attr)) {
        case 1 :
          return $this->_getClassAttr($name);
        case 3 || 4 :
          return $this->_getXMLFileAttr($attr);
        default:
          throw new Exception('The attribute "' . $name . '" does not exist for this station instance.');
      }
    } catch (Exception $e) {
      echo $e;
    }
  }

  /**
   * Get class attribute values: sensors and units. 
   * @param string $name
   * @return string Sensor or unit value
   */
  private function _getClassAttr($name) {
    switch ($name) {
      case 'sensors':
        return $this->_sensors;
      case 'units':
        return $this->_units;
    }
  }

  /**
   * Get Meteohub XML-Data items. Can use short or long tags. 
   * The long tags format defined in "4.4 Port 5559: XML-Data. Meteohub User-Manual".
   *
   * Long tag: actual_rain0_total_mm -> $attr Array([0] => actual, [1] =>rain0, [2] => total, [3] => mm)
   * Short tag: actual_rain_total -> $attr Array([0] => actual, [1] => rain, [2] => total) Others exemples: actual_outside_tempmax
   * 
   * @param string $name Tag name
   * @return array Tag value
   */
  private function _getXMLFileAttr($attr) {
    // If $attr == 4 then get long tag
    if (sizeof($attr) == 4) {
      $sensorName = $this->_sensors->$attr[1];
      $sensor = isset($sensorName) ? $sensorName : $attr[1];
      $unit = $attr[3];
      // else is a short tag special case
    } else if ($attr[1] == 'inside' or $attr[1] == 'outside') {
      $sensor = $this->_sensors->$attr[1];
      $unit = $this->_units->$attr[2];
      // else is a normal short tag
    } else {
      $sensor = $this->_sensors->$attr[1];
      $unit = $this->_units->$attr[1];
    }
    $timeframe = $attr[0];
    $cat = $attr[2];

    // $con is a NODEList object obtained from xpath query
    $con = $this->_xpath->query("/meteohub/data[@timeframe='$timeframe']/item[@sensor='$sensor' and @cat='$cat' and @unit='$unit']");

    // If the attribute name no exists or the connection to XML file failed then return 'N/A' value
    if ($con->length == 0)
      return 'N/A';

    return $con->item(0)->nodeValue;
  }
  
   /**
   * Set sensors.
   * 
   * @param array $sensors  Array of sensors values
   * @see Sensor::__construct()
   */
  public function setSensors($sensors = null) {
    require_once 'Sensor.php';
    $this->_sensors = new Sensor($sensors);
  }

  /**
   * Set units.
   * 
   * @param array $units  Array of unit values
   * @see Unit::__construct()
   */
  public function setUnits($units = null) {
    require_once 'Unit.php';
    $this->_units = new Unit($units);
  }

  /**
   * Create a new DOMDocument && DOMXPath instances.
   * 
   * @param string $xmlDoc URI of the XML document that contains station data-values.
   */
  public function setXPath($xmlDoc) {
    $doc = new DOMDocument();
    
    if (!@$doc->load($xmlDoc)){
      //ENVIAR EMAIL
      
      //return false;
    }
    return $this->_xpath = new DOMXPath($doc);
  }
  
}

